/*
 * gtkcooker: an abstraction layer on top of gtk
 * http://sed.free.fr/gtkcooker
 * Public domain.
 */

#include "convert.h"

#include <stdio.h>
#include <iconv.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

static int inited;
static char *buf;
static int size;
static iconv_t driver;

/* return -1 if error, 0 if ok */
int init_convert(char *encoding)
{
  driver = iconv_open("UTF-8", encoding);
  if (driver == (iconv_t)-1) {
    if (errno == EINVAL)
      fprintf(stderr, "error: encoding %s not supported\n", encoding);
    else
      perror("iconv open error");
    return -1;
  }
  inited = 1;
  return 0;
}

void close_convert(void)
{
  if (size) free(buf);
  if (inited) if (iconv_close(driver) != 0) perror("iconv close error");
  size = 0;
  buf = 0;
  inited = 0;
}

/* return -1 if error, 0 if ok */
static int more_room(void)
{
  char *new;
  if (!buf)
    new = malloc(512);
  else
    new = realloc(buf, size+512);
  if (!new) {
    fprintf(stderr, "out of memory\n");
    return -1;
  }
  size += 512;
  buf = new;
  return 0;
}

/* return 0 if error, a converted string if ok */
char *convert(char *s)
{
  /* this function is barbarian - if it fails because of
   * the out buffer being too small, get more room and
   * start all over again. Pretty barbarian but simple
   * and easy to understand (no silly bugs in there).
   */
  int ss = strlen(s);
  const char *in;
  char *out;
  size_t in_size;
  size_t out_size;

  if (!inited) return s;

  if (size == 0) if (more_room() == -1) return 0;

try_again:
  in = s;
  in_size = ss;
  out = buf;
  out_size = size-1;

  if (iconv(driver, &in, &in_size, &out, &out_size) == (size_t)-1) {
    if (errno == E2BIG) {
      if (more_room() == -1) return 0;
      goto try_again;
    }
    perror("iconv error");
    fprintf(stderr, "are you sure you gave the correct encoding?\n");
    return 0;
  }

  *out = 0;
  return buf;
}
