/*
 * gtkcooker: an abstraction layer on top of gtk
 * http://sed.free.fr/gtkcooker
 * Public domain.
 */

#ifndef _GTK_WIDGET_H_
#define _GTK_WIDGET_H_

#include "string.h"
#include "identifier.h"
#include "list.h"

/* the supported gtk widgets */
#define WINDOW		0
#define LABEL		1
#define BUTTON		2
#define BOX		3
#define MENU_ITEM	4
#define MENU		5
#define IMAGE		6
#define TEXT		7
#define SCROLL		8
#define SEPARATOR	9
#define VIEWPORT	10
#define DRAW		11
#define WIDGET_FILE	12
#define RESIZER		13
#define LIST		14

#define GET_TYPE(x)	((x) & 0xff)

/* gtk_widget properties */
/* GLOBAL: should the gtk widget appear in the structure */
#define IS_GLOBAL(x)		(WIDGET_TYPE(x) & 0x100)
#define SET_GLOBAL(x,y)		do { if (y) WIDGET_TYPE(x) |= 0x100; \
				else WIDGET_TYPE(x) &= ~0x100; } while (0)
#define IS_EXPAND(x)		(WIDGET_TYPE(x) & 0x200)
#define SET_EXPAND(x,y)		do { if (y) WIDGET_TYPE(x) |= 0x200; \
				else WIDGET_TYPE(x) &= ~0x200; } while (0)
#define IS_FILL(x)		(WIDGET_TYPE(x) & 0x400)
#define SET_FILL(x,y)		do { if (y) WIDGET_TYPE(x) |= 0x400; \
				else WIDGET_TYPE(x) &= ~0x400; } while (0)
#define IS_HOMOGENEOUS(x)	(WIDGET_TYPE(x) & 0x800)
#define SET_HOMOGENEOUS(x,y)	do { if (y) WIDGET_TYPE(x) |= 0x800; \
				else WIDGET_TYPE(x) &= ~0x800; } while (0)
#define IS_ACTIVE(x)		(WIDGET_TYPE(x) & 0x1000)
#define SET_ACTIVE(x,y)		do { if (y) WIDGET_TYPE(x) |= 0x1000; \
				else WIDGET_TYPE(x) &= ~0x1000; } while (0)
#define IS_SCROLL(x)		(WIDGET_TYPE(x) & 0x2000)
#define SET_SCROLL(x,y)		do { if (y) WIDGET_TYPE(x) |= 0x2000; \
				else WIDGET_TYPE(x) &= ~0x2000; } while (0)

/* window types */
#define TOPLEVEL	0

/* positions */
#define POSITION_CENTER		0
#define POSITION_MOUSE		1
#define POSITION_DAD		2

/* label types */
#define LABEL_STRING	0
#define LABEL_VARIABLE	1

/* box types */
#define BOX_HORIZONTAL	0
#define BOX_VERTICAL	1

/* menu types */
#define MENU_MENU	0
#define MENU_MENUBAR	1

/* button types */
#define BUTTON_NORMAL	0
#define BUTTON_TOGGLE	1
#define BUTTON_CHECK	2
#define BUTTON_RADIO	3

/* image sources */
#define IMAGE_FILE_STRING	0
#define IMAGE_FILE_IDENTIFIER	1
#define IMAGE_MEMORY		2

/* text types */
#define TEXT_ENTRY		0
#define TEXT_MULTILINES		1

/* text initial text types */
#define TEXT_STRING	0
#define TEXT_VARIABLE	1

/* scroll types */
#define SCROLL_ALWAYS		0
#define SCROLL_AUTOMATIC	1
#define SCROLL_NEVER		2

/* scroll position */
/* if you change the values, check scroll.c:generate_variable_instanciation */
#define SCROLL_LEFT		0
#define SCROLL_RIGHT		1
#define SCROLL_TOP		2
#define SCROLL_BOTTOM		3

/* separator types */
#define SEPARATOR_HORIZONTAL	0
#define SEPARATOR_VERTICAL	1

/* file title type */
#define FILE_TITLE_STRING	0
#define FILE_TITLE_VARIABLE	1

/* resizer types */
#define RESIZER_HORIZONTAL	0
#define RESIZER_VERTICAL	1

typedef int gtk_widget;

#include "widget_driver.h"

typedef struct {
  int type;			/* lower 8 bits: type, higher: properties */
  int border_width;		/* -1 means no border to set */
  int width;			/* -1 means no width to set */
  int height;			/* -1 means no height to set */
  int padding;			/* -1 means no padding (padding will be 0) */
  string font;			/* -1 for no font set */
  /* for background and foreground -2 is used to not propagate values to
   * children widgets
   */
  string background_normal;	/* -1 means not set, -2 for default */
  string background_active;	/* -1 means not set, -2 for default */
  string background_prelight;	/* -1 means not set, -2 for default */
  string background_selected;	/* -1 means not set, -2 for default */
  string background_insensitive;	/* -1 means not set, -2 for default */
  string foreground_normal;	/* -1 means not set, -2 for default */
  string foreground_active;	/* -1 means not set, -2 for default */
  string foreground_prelight;	/* -1 means not set, -2 for default */
  string foreground_selected;	/* -1 means not set, -2 for default */
  string foreground_insensitive;	/* -1 means not set, -2 for default */
  identifier name;
  list signals;			/* <signal name> <function name> <code> ... */
  gtk_widget dad;		/* in "w > x", w is dad of x */
  widget_driver *driver;
  int hscroll_type;
  int vscroll_type;
  int hscroll_position;
  int vscroll_position;
  union {
    struct {
      int type;
      gtk_widget child;
      identifier dad;
      int position;
      int modal;
    } window;
    struct {
      int type;
      int text;			/* string or identifier */
      float halign;
      float valign;
    } label;
    struct {
      int source_type;
      int source;		/* string or identifier */
      float halign;
      float valign;
    } image;
    struct {
      int type;
      gtk_widget child;
      gtk_widget group;		/* for radio button */
    } button;
    struct {
      int type;
      list children;		/* list of gtk_widget */
      int spacing;
    } box;
    struct {
      gtk_widget child;
      gtk_widget submenu;	/* -1 for no submenu */
    } menu_item;
    struct {
      int type;
      list children;		/* list of gtk_widget (value:-2=>separator) */
    } menu;
    struct {
      int type;
      int width_chars;
      int initial_text;
      int initial_text_type;
    } text;
    struct {
      gtk_widget child;
    } scroll;
    struct {
      int type;
    } separator;
    struct viewport {
      gtk_widget child;
    } viewport;
    struct draw {
    } draw;
    struct file {
      int title_type;
      int title;		/* string or identifier */
      int position;
    } file;
    struct resizer {
      int type;
      gtk_widget first_child;
      gtk_widget second_child;
    } resizer;
    struct list {
      identifier name;
      list data;		/* list of strings */
    } list;
  } s;
} _gtk_widget;

gtk_widget new_gtk_widget(void);
void close_gtk_widget(void);
identifier new_name(void);
void gtk_widget_copy(gtk_widget, gtk_widget);
_gtk_widget *get_gtk_widget(gtk_widget);

#define WIDGET_TYPE(x) get_gtk_widget(x)->type
#define WIDGET_NAME(x) get_gtk_widget(x)->name
#define WIDGET_SIGNALS(x) get_gtk_widget(x)->signals
#define WIDGET_DRIVER(x) get_gtk_widget(x)->driver
#define WIDGET_BORDER_WIDTH(x) get_gtk_widget(x)->border_width
#define WIDGET_WIDTH(x) get_gtk_widget(x)->width
#define WIDGET_HEIGHT(x) get_gtk_widget(x)->height
#define WIDGET_PADDING(x) get_gtk_widget(x)->padding
#define WIDGET_FONT(x) get_gtk_widget(x)->font
#define WIDGET_FOREGROUND_NORMAL(x) get_gtk_widget(x)->foreground_normal
#define WIDGET_FOREGROUND_ACTIVE(x) get_gtk_widget(x)->foreground_active
#define WIDGET_FOREGROUND_PRELIGHT(x) get_gtk_widget(x)->foreground_prelight
#define WIDGET_FOREGROUND_SELECTED(x) get_gtk_widget(x)->foreground_selected
#define WIDGET_FOREGROUND_INSENSITIVE(x) \
		get_gtk_widget(x)->foreground_insensitive
#define WIDGET_BACKGROUND_NORMAL(x) get_gtk_widget(x)->background_normal
#define WIDGET_BACKGROUND_ACTIVE(x) get_gtk_widget(x)->background_active
#define WIDGET_BACKGROUND_PRELIGHT(x) get_gtk_widget(x)->background_prelight
#define WIDGET_BACKGROUND_SELECTED(x) get_gtk_widget(x)->background_selected
#define WIDGET_BACKGROUND_INSENSITIVE(x) \
		get_gtk_widget(x)->background_insensitive
#define WIDGET_HSCROLL_POSITION(x) get_gtk_widget(x)->hscroll_position
#define WIDGET_VSCROLL_POSITION(x) get_gtk_widget(x)->vscroll_position
#define WIDGET_HSCROLL_TYPE(x) get_gtk_widget(x)->hscroll_type
#define WIDGET_VSCROLL_TYPE(x) get_gtk_widget(x)->vscroll_type

#define WINDOW_TYPE(x) get_gtk_widget(x)->s.window.type
#define WINDOW_CHILD(x) get_gtk_widget(x)->s.window.child
#define WINDOW_DAD(x) get_gtk_widget(x)->s.window.dad
#define WINDOW_POSITION(x) get_gtk_widget(x)->s.window.position
#define WINDOW_MODAL(x) get_gtk_widget(x)->s.window.modal

#define BOX_TYPE(x) get_gtk_widget(x)->s.box.type
#define BOX_CHILDREN(x) get_gtk_widget(x)->s.box.children
#define BOX_SPACING(x) get_gtk_widget(x)->s.box.spacing

#define BUTTON_TYPE(x) get_gtk_widget(x)->s.button.type
#define BUTTON_CHILD(x) get_gtk_widget(x)->s.button.child
#define BUTTON_GROUP(x) get_gtk_widget(x)->s.button.group

#define LABEL_TYPE(x) get_gtk_widget(x)->s.label.type
#define LABEL_TEXT(x) get_gtk_widget(x)->s.label.text
#define LABEL_HALIGN(x) get_gtk_widget(x)->s.label.halign
#define LABEL_VALIGN(x) get_gtk_widget(x)->s.label.valign

#define IMAGE_SOURCE_TYPE(x) get_gtk_widget(x)->s.image.source_type
#define IMAGE_SOURCE(x) get_gtk_widget(x)->s.image.source
#define IMAGE_HALIGN(x) get_gtk_widget(x)->s.image.halign
#define IMAGE_VALIGN(x) get_gtk_widget(x)->s.image.valign

#define MENU_ITEM_CHILD(x) get_gtk_widget(x)->s.menu_item.child
#define MENU_ITEM_SUBMENU(x) get_gtk_widget(x)->s.menu_item.submenu

#define MENU_TYPE(x) get_gtk_widget(x)->s.menu.type
#define MENU_CHILDREN(x) get_gtk_widget(x)->s.menu.children

#define TEXT_TYPE(x) get_gtk_widget(x)->s.text.type
#define TEXT_INITIAL_TEXT(x) get_gtk_widget(x)->s.text.initial_text
#define TEXT_INITIAL_TEXT_TYPE(x) get_gtk_widget(x)->s.text.initial_text_type

#define SCROLL_CHILD(x) get_gtk_widget(x)->s.scroll.child

#define SEPARATOR_TYPE(x) get_gtk_widget(x)->s.separator.type

#define VIEWPORT_CHILD(x) get_gtk_widget(x)->s.viewport.child

#define FILE_TITLE_TYPE(x) get_gtk_widget(x)->s.file.title_type
#define FILE_TITLE(x) get_gtk_widget(x)->s.file.title
#define FILE_POSITION(x) get_gtk_widget(x)->s.file.position

#define RESIZER_TYPE(x) get_gtk_widget(x)->s.resizer.type
#define RESIZER_FIRST_CHILD(x) get_gtk_widget(x)->s.resizer.first_child
#define RESIZER_SECOND_CHILD(x) get_gtk_widget(x)->s.resizer.second_child

#define LIST_NAME(x) get_gtk_widget(x)->s.list.name
#define LIST_DATA(x) get_gtk_widget(x)->s.list.data

#endif /* _GTK_WIDGET_H_ */
