/*
 * gtkcooker: an abstraction layer on top of gtk
 * http://sed.free.fr/gtkcooker
 * Public domain.
 */

#include <string.h>

#include "widget.h"
#include "generate.h"
#include "files.h"
#include "keywords.h"
#include "code.h"
#include "globals.h"

extern widget_driver draw_widget_driver;

/* -1 error, 0 ok */
static int first_pass_translation(widget w, widget_element e)
{
  gtk_widget draw;
  list l;
  int i;
  int n;

  draw = new_gtk_widget();
  if (draw == -1) return -1;

  if (widget_add_gtk_widget(w, draw) == -1)
    return -1;

  widget_element_set_gtk_widget(w, e, draw);

  WIDGET_TYPE(draw) = DRAW;
  WIDGET_NAME(draw) = widget_element_name(w, e);
  WIDGET_DRIVER(draw) = &draw_widget_driver;
  if (!default_local)
    SET_GLOBAL(draw, 1);

  /* properties */
  l = widget_element_properties(w, e);
  n = list_size(l);
  for (i=0; i<n; i+=3) {
    keyword id = (keyword)list_get_int(l, i);
    int type = list_get_int(l, i+1);

    switch (id) {
      CASE_PROPERTY_BORDER(draw, l, i)
      CASE_PROPERTY_WIDTH(draw)
      CASE_PROPERTY_HEIGHT(draw)
      CASE_PROPERTY_GROW(draw, l, i)
      CASE_PROPERTY_PADDING(draw, l, i)
      CASE_PROPERTY_HOMOGENEOUS(draw, l, i)
      CASE_PROPERTY_GLOBAL(draw)
      CASE_PROPERTY_FOREGROUND_AND_BACKGROUND(draw)
      CASE_PROPERTY_SCROLL(draw)
      CASE_PROPERTY_HSCROLL(draw)
      CASE_PROPERTY_VSCROLL(draw)
      CASE_PROPERTY_EXPOSE(draw)
      CASE_PROPERTY_POINTER(draw)
      CASE_PROPERTY_ENTER(draw)
      CASE_PROPERTY_LEAVE(draw)
      CASE_PROPERTY_KEY(draw)
      CASE_PROPERTY_FOCUS(draw)
      CASE_PROPERTY_DEFAULT
    }
  }

  return 0;
}

/* -1 error, 0 ok */
static int generate_struct(gtk_widget gw)
{
  if (!IS_GLOBAL(gw)) return 0;

  if (hide_gtk)
    PUT_H("  /* GtkWidget* */ void *");
  else
    PUT_H("  GtkWidget *");
  PUT_H(get_identifier(WIDGET_NAME(gw)));
  PUT_H(";\n");

  return 0;
}

/* -1 error, 0 ok */
static int generate_signal_handlers(widget w, gtk_widget gw)
{
  int i;
  list l = WIDGET_SIGNALS(gw);
  for (i=0; i<list_size(l); i+=3) {
    keyword id = (keyword)list_get_int(l, i);
    switch (id) {
      CASE_GENERATE_SIGNAL_HANDLER_EXPOSE
      CASE_GENERATE_SIGNAL_HANDLER_POINTER
      CASE_GENERATE_SIGNAL_HANDLER_ENTER
      CASE_GENERATE_SIGNAL_HANDLER_LEAVE
      CASE_GENERATE_SIGNAL_HANDLER_KEY
      CASE_GENERATE_SIGNAL_HANDLER_FOCUS
      CASE_GENERATE_SIGNAL_HANDLER_DEFAULT
    }
  }
  return 0;
}

/* -1 error, 0 ok */
static int generate_variable_declaration(gtk_widget gw)
{
  if (IS_GLOBAL(gw)) return 0;

  PUT_C("  GtkWidget *");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C(";\n");

  return 0;
}

/* -1 error, 0 ok */
static int generate_variable_instanciation(gtk_widget gw)
{
  PUT_C("  ");
  if (IS_GLOBAL(gw)) PUT_C("this->");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C(" = gtk_drawing_area_new();\n");

  GENERATE_ADD_EVENT_POINTER
  GENERATE_ADD_EVENT_ENTER
  GENERATE_ADD_EVENT_LEAVE
  GENERATE_ADD_EVENT_KEY

  GENERATE_BORDER_WIDTH

  if (WIDGET_WIDTH(gw) == -1) WIDGET_WIDTH(gw) = 1;
  if (WIDGET_HEIGHT(gw) == -1) WIDGET_HEIGHT(gw) = 1;

  GENERATE_WIDTH_AND_HEIGHT

  GENERATE_FOREGROUND_AND_BACKGROUND

  return 0;
}

/* -1 error, 0 ok */
static int generate_signals(gtk_widget gw)
{
  int i;
  list l = WIDGET_SIGNALS(gw);

  for (i=0; i<list_size(l); i+=3) {
    keyword id = (keyword)list_get_int(l, i);

    switch (id) {
      CASE_GENERATE_SIGNAL_EXPOSE
      CASE_GENERATE_SIGNAL_POINTER
      CASE_GENERATE_SIGNAL_ENTER
      CASE_GENERATE_SIGNAL_LEAVE
      CASE_GENERATE_SIGNAL_KEY
      CASE_GENERATE_SIGNAL_FOCUS
      CASE_GENERATE_SIGNAL_DEFAULT
    }
  }
  if (list_size(l)) PUT_C("\n");

  return 0;
}

widget_driver draw_widget_driver = {
  .name = "draw",
  .first_pass_translation = first_pass_translation,
  .second_pass_translation = 0,
  .populate_properties = 0,
  .generate_struct = generate_struct,
  .generate_signal_handlers = generate_signal_handlers,
  .generate_variable_declaration = generate_variable_declaration,
  .generate_variable_instanciation = generate_variable_instanciation,
  .generate_pack = 0,
  .generate_post_pack = 0,
  .generate_signals = generate_signals
};
