/*
 * gtkcooker: an abstraction layer on top of gtk
 * http://sed.free.fr/gtkcooker
 * Public domain.
 */

#include <string.h>

#include "widget.h"
#include "generate.h"
#include "files.h"
#include "widget.h"
#include "keywords.h"
#include "code.h"
#include "globals.h"

extern widget_driver text_widget_driver;

/* -1 error, 0 ok */
static int first_pass_translation(widget w, widget_element e)
{
  gtk_widget text;
  list l;
  int n;
  int i;
  int activate_property_set = 0;

  if (list_size(widget_element_pack(w, e))) {
    fprintf(stderr, "packed text not supported (text %s)\n",
            get_identifier(widget_element_name(w, e)));
    return -1;
  }

  l = widget_element_arguments(w, e);
  if (list_size(l) && (list_size(l) != 2 ||
        (list_get_int(l, 0) != WA_STRING &&
         list_get_int(l, 0) != WA_IDENTIFIER))) {
    fprintf(stderr,
            "text %s not properly declared. You must do something like:\n",
            get_identifier(widget_element_name(w, e)));
    fprintf(stderr, "  text %s(\"text' initial text\");\n",
            get_identifier(widget_element_name(w, e)));
    fprintf(stderr, "or:\n  text %s;\n",
            get_identifier(widget_element_name(w, e)));
    fprintf(stderr, "or:\n  text %s(text);\n",
            get_identifier(widget_element_name(w, e)));
    fprintf(stderr, "where 'text' is a NULLZ char * argument\n");
    return -1;
  }

  text = new_gtk_widget();
  if (text == -1) return -1;

  if (widget_add_gtk_widget(w, text) == -1)
    return -1;

  widget_element_set_gtk_widget(w, e, text);

  WIDGET_TYPE(text) = TEXT;
  WIDGET_NAME(text) = widget_element_name(w, e);
  WIDGET_DRIVER(text) = &text_widget_driver;
  if (!default_local)
    SET_GLOBAL(text, 1);
  TEXT_TYPE(text) = TEXT_ENTRY;
  if (list_size(l) == 2) {
    if (list_get_int(l, 0) == WA_STRING)
      TEXT_INITIAL_TEXT_TYPE(text) = TEXT_STRING;
    else
      TEXT_INITIAL_TEXT_TYPE(text) = TEXT_VARIABLE;
    TEXT_INITIAL_TEXT(text) = (string)list_get_int(l, 1);
  }

  /* properties */
  l = widget_element_properties(w, e);
  n = list_size(l);
  for (i=0; i<n; i+=3) {
    keyword id = (keyword)list_get_int(l, i);
    int type = list_get_int(l, i+1);

    switch (id) {
      CASE_PROPERTY_TEXT_TYPE(text)
      CASE_PROPERTY_WIDTH(text)
      CASE_PROPERTY_HEIGHT(text)
      CASE_PROPERTY_GROW(text, l, i)
      CASE_PROPERTY_PADDING(text, l, i)
      CASE_PROPERTY_GLOBAL(text)
      CASE_PROPERTY_SCROLL(text)
      CASE_PROPERTY_HSCROLL(text)
      CASE_PROPERTY_VSCROLL(text)
      CASE_PROPERTY_TEXT_ACTIVATE(text)
      CASE_PROPERTY_DEFAULT
    }
  }

  if (TEXT_TYPE(text) != TEXT_ENTRY && activate_property_set) {
    fprintf(stderr,
            "'activate' property not allowed for text %s.\n",
            get_identifier(widget_element_name(w, e)));
    return -1;
  }

  return 0;
}

#if 0
/* -1 error, 0 ok */
static int second_pass_translation(widget w, widget_element e)
{
  return 0;
}
#endif

#if 0
/* -1 error, 0 ok */
static int populate_properties(gtk_widget gw)
{
  return 0;
}
#endif

/* -1 error, 0 ok */
static int generate_struct(gtk_widget gw)
{
  if (!IS_GLOBAL(gw)) return 0;

  if (hide_gtk)
    PUT_H("  /* GtkWidget* */ void *");
  else
    PUT_H("  GtkWidget *");
  PUT_H(get_identifier(WIDGET_NAME(gw)));
  PUT_H(";\n");

  return 0;
}

/* -1 error, 0 ok */
static int generate_signal_handlers(widget w, gtk_widget gw)
{
  int i;
  list l = WIDGET_SIGNALS(gw);
  for (i=0; i<list_size(l); i+=3) {
    keyword id = (keyword)list_get_int(l, i);
    switch (id) {
      CASE_GENERATE_SIGNAL_HANDLER_TEXT_ACTIVATE
      CASE_GENERATE_SIGNAL_HANDLER_DEFAULT
    }
  }
  return 0;
}

/* -1 error, 0 ok */
static int generate_variable_declaration(gtk_widget gw)
{
  if (IS_GLOBAL(gw)) return 0;

  PUT_C("  GtkWidget *");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C(";\n");

  return 0;
}

/* -1 error, 0 ok */
static int generate_variable_instanciation(gtk_widget gw)
{
  PUT_C("  ");
  if (IS_GLOBAL(gw)) PUT_C("this->");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  if (TEXT_TYPE(gw) == TEXT_ENTRY)
    PUT_C(" = gtk_entry_new();\n");
  else
    PUT_C(" = gtk_text_view_new();\n");

  if (TEXT_INITIAL_TEXT(gw) != -1) {
    if (TEXT_TYPE(gw) == TEXT_ENTRY) {
      PUT_C("  gtk_entry_set_text(GTK_ENTRY(");
      if (IS_GLOBAL(gw)) PUT_C("this->");
      PUT_C(get_identifier(WIDGET_NAME(gw)));
      PUT_C("),\n    ");
      if (TEXT_INITIAL_TEXT_TYPE(gw) == TEXT_STRING) {
        PUT_C("\"");
        PUT_C(get_string(TEXT_INITIAL_TEXT(gw)));
        PUT_C("\"");
      } else
        PUT_C(get_identifier(TEXT_INITIAL_TEXT(gw)));
      PUT_C(");\n");
    } else {
      PUT_C("  gtk_text_buffer_set_text(gtk_text_view_get_buffer(\n");
      PUT_C("    GTK_TEXT_VIEW(");
      if (IS_GLOBAL(gw)) PUT_C("this->");
      PUT_C(get_identifier(WIDGET_NAME(gw)));
      PUT_C(")),\n    ");
      if (TEXT_INITIAL_TEXT_TYPE(gw) == TEXT_STRING) {
        PUT_C("\"");
        PUT_C(get_string(TEXT_INITIAL_TEXT(gw)));
        PUT_C("\"");
      } else
        PUT_C(get_identifier(TEXT_INITIAL_TEXT(gw)));
      PUT_C(", -1);\n");
    }
  }

  GENERATE_WIDTH_AND_HEIGHT
  GENERATE_BORDER_WIDTH_TEXT
  GENERATE_FONT
  GENERATE_FOREGROUND_AND_BACKGROUND

  return 0;
}

#if 0
/* -1 error, 0 ok */
static int generate_pack(gtk_widget gw)
{
  return 0;
}
#endif

#if 0
/* -1 error, 0 ok */
static int generate_post_pack(gtk_widget gw)
{
  return 0;
}
#endif

/* -1 error, 0 ok */
static int generate_signals(gtk_widget gw)
{
  int i;
  list l = WIDGET_SIGNALS(gw);
  for (i=0; i<list_size(l); i+=3) {
    keyword id = (keyword)list_get_int(l, i);
    switch (id) {
      CASE_GENERATE_SIGNAL_TEXT_ACTIVATE
      CASE_GENERATE_SIGNAL_DEFAULT
    }
  }
  if (list_size(l)) PUT_C("\n");

  return 0;
}

widget_driver text_widget_driver = {
  .name = "text",
  .first_pass_translation = first_pass_translation,
  .second_pass_translation = 0,
  .populate_properties = 0,
  .generate_struct = generate_struct,
  .generate_signal_handlers = generate_signal_handlers,
  .generate_variable_declaration = generate_variable_declaration,
  .generate_variable_instanciation = generate_variable_instanciation,
  .generate_pack = 0,
  .generate_post_pack = 0,
  .generate_signals = generate_signals
};
