/*
 * gtkcooker: an abstraction layer on top of gtk
 * http://sed.free.fr/gtkcooker
 * Public domain.
 */

#include <string.h>

#include "widget.h"
#include "generate.h"
#include "files.h"
#include "keywords.h"
#include "code.h"
#include "globals.h"

extern widget_driver window_widget_driver;
extern widget_driver box_widget_driver;
extern widget_driver separator_widget_driver;
extern widget_driver resizer_widget_driver;

/* -1 error, 0 ok */
static int first_pass_translation(widget w, widget_element e)
{
  gtk_widget win;
  gtk_widget box;
  list l;
  int i;
  int n;

  win = new_gtk_widget();
  box = new_gtk_widget();
  if (win == -1 || box == -1) return -1;

  if (widget_add_gtk_widget(w, win) == -1 ||
      widget_add_gtk_widget(w, box) == -1)
    return -1;

  widget_element_set_gtk_widget(w, e, win);

  WIDGET_TYPE(win) = WINDOW;
  WIDGET_NAME(win) = widget_element_name(w, e);
  WIDGET_DRIVER(win) = &window_widget_driver;
  if (!default_local)
    SET_GLOBAL(win, 1);
  WINDOW_TYPE(win) = TOPLEVEL;
  WINDOW_CHILD(win) = box;

  WIDGET_TYPE(box) = BOX;
  WIDGET_NAME(box) = new_name();
  if (WIDGET_NAME(box) == -1) return -1;
  WIDGET_DRIVER(box) = &box_widget_driver;
  BOX_TYPE(box) = BOX_VERTICAL;

  /* properties */
  l = widget_element_properties(w, e);
  n = list_size(l);
  for (i=0; i<n; i+=3) {
    keyword id = (keyword)list_get_int(l, i);
    int type = list_get_int(l, i+1);

    switch (id) {
      CASE_PROPERTY_BORDER(win, l, i)
      CASE_PROPERTY_WIDTH(win)
      CASE_PROPERTY_HEIGHT(win)
      CASE_PROPERTY_BOX_SPACING(box, l, i)
      CASE_PROPERTY_HOMOGENEOUS(box, l, i)
      CASE_PROPERTY_BOX_DIRECTION(box)
      CASE_PROPERTY_GLOBAL(win)
      CASE_PROPERTY_FONT(win)
      CASE_PROPERTY_FOREGROUND_AND_BACKGROUND(win)
      CASE_PROPERTY_WINDOW_DAD(win)
      CASE_PROPERTY_WINDOW_POSITION(win)
      CASE_PROPERTY_WINDOW_MODAL(win)
      CASE_PROPERTY_SCROLL(box)
      CASE_PROPERTY_HSCROLL(box)
      CASE_PROPERTY_VSCROLL(box)
      CASE_PROPERTY_DELETE(win)
      CASE_PROPERTY_DEFAULT
    }
  }

  return 0;
}

/* -1 error, 0 ok */
static int second_pass_translation(widget w, widget_element e)
{
  int i;
  int j;
  int n1;
  int n2;
  list l;
  list l2;
  gtk_widget box = WINDOW_CHILD(widget_element_gtk_widget(w, e));
  gtk_widget box_in;
  int nb_resizers;

  BOX_CHILDREN(box) = new_list();
  if (BOX_CHILDREN(box) == -1) return -1;

  l = widget_element_pack(w, e);
  n1 = list_size(l);

  for (i=0; i<n1; i++) {
    box_in = new_gtk_widget();
    if (box_in == -1) return -1;

    if (widget_add_gtk_widget(w, box_in) == -1) return -1;

    if (list_add_int(BOX_CHILDREN(box), box_in) == -1) return -1;

    WIDGET_TYPE(box_in) = BOX;
    WIDGET_NAME(box_in) = new_name();
    if (WIDGET_NAME(box_in) == -1) return -1;
    WIDGET_DRIVER(box_in) = &box_widget_driver;
    BOX_TYPE(box_in) =
        BOX_TYPE(box) == BOX_VERTICAL ? BOX_HORIZONTAL : BOX_VERTICAL;

    BOX_CHILDREN(box_in) = new_list();
    if (BOX_CHILDREN(box_in) == -1) return -1;

    /* packing elements */
    l2 = (list)list_get_int(l, i);
    n2 = list_size(l2) - 1;

    /* resizers have to be handled separately */
    /* only one resizer allowed and left and right stuff must be present */
    nb_resizers = 0;
    for (j=0; j<n2; j++)
      if (list_get_int(l2, j+1) == -4) {
        nb_resizers++;
        if (j == 0 || j == n2-1) {
          fprintf(stderr, "error with resizer with widget %s\n",
                  widget_name(w));
          fprintf(stderr, "it cannot be the first or last in the packing\n");
          return -1;
        }
      }
    if (nb_resizers > 1) {
      fprintf(stderr, "error with resizer with widget %s\n", widget_name(w));
      fprintf(stderr,
              "only one resizer in each pack instruction is allowed\n");
      return -1;
    }

    if (nb_resizers) {
      gtk_widget first_box;
      gtk_widget second_box;
      gtk_widget wson = new_gtk_widget();
      if (wson == -1) return -1;
      if (widget_add_gtk_widget(w, wson) == -1) return -1;
      WIDGET_TYPE(wson) = RESIZER;
      WIDGET_NAME(wson) = new_name();
      if (WIDGET_NAME(wson) == -1) return -1;
      WIDGET_DRIVER(wson) = &resizer_widget_driver;
      RESIZER_TYPE(wson) =
          BOX_TYPE(box_in) == BOX_VERTICAL ?
              RESIZER_VERTICAL : RESIZER_HORIZONTAL;
      SET_EXPAND(wson, 1);
      SET_FILL(wson, 1);
      if (list_add_int(BOX_CHILDREN(box_in), wson) == -1) return -1;

      first_box = new_gtk_widget();
      if (first_box == -1) return -1;

      if (widget_add_gtk_widget(w, first_box) == -1) return -1;

      RESIZER_FIRST_CHILD(wson) = first_box;

      WIDGET_TYPE(first_box) = BOX;
      WIDGET_NAME(first_box) = new_name();
      if (WIDGET_NAME(first_box) == -1) return -1;
      WIDGET_DRIVER(first_box) = &box_widget_driver;
      BOX_TYPE(first_box) =
          BOX_TYPE(box) == BOX_VERTICAL ? BOX_HORIZONTAL : BOX_VERTICAL;

      BOX_CHILDREN(first_box) = new_list();
      if (BOX_CHILDREN(first_box) == -1) return -1;

      second_box = new_gtk_widget();
      if (second_box == -1) return -1;

      if (widget_add_gtk_widget(w, second_box) == -1) return -1;

      RESIZER_SECOND_CHILD(wson) = second_box;

      WIDGET_TYPE(second_box) = BOX;
      WIDGET_NAME(second_box) = new_name();
      if (WIDGET_NAME(second_box) == -1) return -1;
      WIDGET_DRIVER(second_box) = &box_widget_driver;
      BOX_TYPE(second_box) =
          BOX_TYPE(box) == BOX_VERTICAL ? BOX_HORIZONTAL : BOX_VERTICAL;

      BOX_CHILDREN(second_box) = new_list();
      if (BOX_CHILDREN(second_box) == -1) return -1;

      /* add children in correct box */
      for (j = 0; j < n2; j++) {
        gtk_widget wson;
        if (list_get_int(l2, j+1) == -4) break;
        wson = widget_element_gtk_widget(w,
                 (widget_element)list_get_int(l2, j + 1));
        if (list_add_int(BOX_CHILDREN(first_box), wson) == -1) return -1;
      }
      for (j++; j < n2; j++) {
        gtk_widget wson;
        wson = widget_element_gtk_widget(w,
                 (widget_element)list_get_int(l2, j + 1));
        if (list_add_int(BOX_CHILDREN(second_box), wson) == -1) return -1;
      }
    }

    if (!nb_resizers) for (j=0; j<n2; j++) {
      gtk_widget wson;
      /* if it's a separator, let's create it */
      if (list_get_int(l2, j+1) == -3) {
        wson = new_gtk_widget();
        if (wson == -1) return -1;
        if (widget_add_gtk_widget(w, wson) == -1) return -1;
        WIDGET_TYPE(wson) = SEPARATOR;
        WIDGET_NAME(wson) = new_name();
        if (WIDGET_NAME(wson) == -1) return -1;
        WIDGET_DRIVER(wson) = &separator_widget_driver;
        if (n2 == 1) {
          SEPARATOR_TYPE(wson) =
              BOX_TYPE(box) == BOX_VERTICAL ?
                  SEPARATOR_HORIZONTAL : SEPARATOR_VERTICAL;
          SET_EXPAND(wson, 1);
          SET_FILL(wson, 1);
        } else
          SEPARATOR_TYPE(wson) =
              BOX_TYPE(box) == BOX_VERTICAL ?
                  SEPARATOR_VERTICAL : SEPARATOR_HORIZONTAL;
      } else {
        wson = widget_element_gtk_widget(w,
                 (widget_element)list_get_int(l2, j + 1));
      }
      if (list_add_int(BOX_CHILDREN(box_in), wson) == -1) return -1;
    }

    /* properties of containing box */
    l2 = (list)list_get_int(l2, 0);
    n2 = l2 == -1 ? 0 : list_size(l2);
    for (j = 0; j < n2; j+=3) {
      keyword id = (keyword)list_get_int(l2, j);
      int type = list_get_int(l2, j+1);

      switch (id) {
        CASE_PROPERTY_BOX_SPACING(box_in, l2, j)
        CASE_PROPERTY_HOMOGENEOUS(box_in, l2, j)
        CASE_PROPERTY_GROW(box_in, l2, j)
        CASE_PROPERTY_PADDING(box_in, l2, j)
        CASE_PROPERTY_BORDER(box_in, l2, j)
        CASE_PROPERTY_DEFAULT_IN_PACKING
      }
    }
  }

  return 0;
}

/* -1 error, 0 ok */
static int populate_properties(gtk_widget gw)
{
  int has_to_populate = 0;
  gtk_widget sw = WINDOW_CHILD(gw);
  widget_driver *sd = WIDGET_DRIVER(sw);

  POPULATE_PROPERTIES

  if (has_to_populate)
    if (sd->populate_properties)
      if (sd->populate_properties(sw) == -1) return -1;

  return 0;
}

/* -1 error, 0 ok */
static int generate_struct(gtk_widget gw)
{
  if (!IS_GLOBAL(gw)) return 0;

  if (hide_gtk)
    PUT_H("  /* GtkWidget* */ void *");
  else
    PUT_H("  GtkWidget *");
  PUT_H(get_identifier(WIDGET_NAME(gw)));
  PUT_H(";\n");

  return 0;
}

/* -1 error, 0 ok */
static int generate_signal_handlers(widget w, gtk_widget gw)
{
  int i;
  list l = WIDGET_SIGNALS(gw);
  for (i=0; i<list_size(l); i+=3) {
    keyword id = (keyword)list_get_int(l, i);
    switch (id) {
      CASE_GENERATE_SIGNAL_HANDLER_DELETE
      CASE_GENERATE_SIGNAL_HANDLER_DEFAULT
    }
  }
  return 0;
}

/* -1 error, 0 ok */
static int generate_variable_declaration(gtk_widget gw)
{
  if (IS_GLOBAL(gw)) return 0;

  PUT_C("  GtkWidget *");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C(";\n");

  return 0;
}

/* -1 error, 0 ok */
static int generate_variable_instanciation(gtk_widget gw)
{
  PUT_C("  ");
  if (IS_GLOBAL(gw)) PUT_C("this->");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C(" = gtk_window_new(");
  switch(WINDOW_TYPE(gw)) {
    case TOPLEVEL: PUT_C("GTK_WINDOW_TOPLEVEL");
    default: break;
  }
  PUT_C(");\n");

  GENERATE_BORDER_WIDTH
  GENERATE_WIDTH_AND_HEIGHT
  GENERATE_FOREGROUND_AND_BACKGROUND
  GENERATE_WINDOW_DAD
  GENERATE_WINDOW_POSITION
  GENERATE_WINDOW_MODAL

  /* we want to allow the user to resize the window as small as she wants */
  PUT_C("  {\n    GdkGeometry _geo; _geo.min_width = _geo.min_height = 1;\n");
  PUT_C("    gtk_window_set_geometry_hints(GTK_WINDOW(");
  if (IS_GLOBAL(gw)) PUT_C("this->");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C("),\n        ");
  if (hide_gtk)
    PUT_C("(GtkWidget *)");
  if (IS_GLOBAL(gw)) PUT_C("this->");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C(", &_geo, GDK_HINT_MIN_SIZE);\n  }\n");

  return 0;
}

/* -1 error, 0 ok */
static int generate_pack(gtk_widget gw)
{
  gtk_widget sw = WINDOW_CHILD(gw);
  PUT_C("  gtk_container_add(GTK_CONTAINER(");
  if (IS_GLOBAL(gw)) PUT_C("this->");
  PUT_C(get_identifier(WIDGET_NAME(gw)));
  PUT_C("), ");
  if (hide_gtk)
    PUT_C("(GtkWidget *)");
  if (IS_GLOBAL(sw)) PUT_C("this->");
  PUT_C(get_identifier(WIDGET_NAME(sw)));
  PUT_C(");\n");

  return 0;
}

/* -1 error, 0 ok */
static int generate_signals(gtk_widget gw)
{
  int i;
  list l = WIDGET_SIGNALS(gw);

  for (i=0; i<list_size(l); i+=3) {
    keyword id = (keyword)list_get_int(l, i);

    switch (id) {
      CASE_GENERATE_SIGNAL_DELETE
      CASE_GENERATE_SIGNAL_DEFAULT
    }
  }
  if (list_size(l)) PUT_C("\n");

  return 0;
}

widget_driver window_widget_driver = {
  .name = "window",
  .first_pass_translation = first_pass_translation,
  .second_pass_translation = second_pass_translation,
  .populate_properties = populate_properties,
  .generate_struct = generate_struct,
  .generate_signal_handlers = generate_signal_handlers,
  .generate_variable_declaration = generate_variable_declaration,
  .generate_variable_instanciation = generate_variable_instanciation,
  .generate_pack = generate_pack,
  .generate_post_pack = 0,
  .generate_signals = generate_signals
};
